module OkStd.Environment
  ( getArgs,
    getProgName,
    lookupEnv,
  )
where

import Control.Monad.Trans (MonadIO, liftIO)
import Data.Function (($))
import Data.Functor (fmap, (<$>))
import Data.Maybe (Maybe)
import qualified Data.Text as T
import qualified System.Environment as E

-- | Returns a list of program's command line arguments (not including the
-- program name)
getArgs :: MonadIO m => m [T.Text]
getArgs =
  fmap T.pack <$> liftIO E.getArgs

-- | Returns the name of the program as it was invoked.
getProgName :: MonadIO m => m T.Text
getProgName =
  T.pack <$> liftIO E.getProgName

-- | Returns the value of the specified environment variable
lookupEnv :: MonadIO m => T.Text -> m (Maybe T.Text)
lookupEnv var =
  fmap T.pack <$> liftIO (E.lookupEnv $ T.unpack var)
