{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE GeneralizedNewtypeDeriving #-}

module OkStd.Logger
  ( LogStr,
    ToLogStr (toLogStr),
    logM,
  )
where

import Data.ByteString (ByteString)
import qualified Data.ByteString.Builder as Builder
import qualified Data.ByteString.Lazy as LBS
import Data.Char (Char)
import Data.Function (id, ($), (.))
import Data.Int (Int)
import Data.Monoid (Monoid)
import Data.Semigroup (Semigroup (..))
import Data.String (IsString)
import Data.Text (Text)
import Data.Text.Encoding (encodeUtf8)
import System.IO

newtype LogStr = LogStr Builder.Builder
  deriving (Semigroup, Monoid, IsString)

class ToLogStr a where
  toLogStr :: a -> LogStr

instance ToLogStr LogStr where
  {-# INLINE toLogStr #-}
  toLogStr = id

instance ToLogStr ByteString where
  {-# INLINE toLogStr #-}
  toLogStr = LogStr . Builder.byteString

instance ToLogStr LBS.ByteString where
  {-# INLINE toLogStr #-}
  toLogStr = LogStr . Builder.lazyByteString

instance ToLogStr [Char] where
  {-# INLINE toLogStr #-}
  toLogStr = LogStr . Builder.stringUtf8

instance ToLogStr Text where
  {-# INLINE toLogStr #-}
  toLogStr = LogStr . Builder.byteString . encodeUtf8

instance ToLogStr Int where
  {-# INLINE toLogStr #-}
  toLogStr = LogStr . Builder.intDec

-- | Prints log message to stderr
logM :: LogStr -> IO ()
logM (LogStr b) =
  LBS.hPut stderr $ Builder.toLazyByteString $ b <> "\n"