{-# LANGUAGE FlexibleInstances #-}

module OkStd.Print
  ( Print,
    hPutStr,
    hPutStrLn,
    putStr,
    putStrLn,
    putErrLn,
    putText,
    shout,
  )
where

import Control.Monad.Trans (MonadIO, liftIO)
import Data.ByteString (ByteString)
import qualified Data.ByteString.Char8 as ByteString
import Data.Char (Char)
import Data.Function ((.))
import Data.Text (Text)
import qualified Data.Text.IO as Text
import System.IO (Handle, stderr, stdout)
import qualified System.IO as SIO

class Print a where
  hPutStr :: MonadIO m => Handle -> a -> m ()
  hPutStrLn :: MonadIO m => Handle -> a -> m ()

  putStr :: MonadIO m => a -> m ()
  putStr = hPutStr stdout

  putStrLn :: MonadIO m => a -> m ()
  putStrLn = hPutStrLn stdout

  putErrLn :: MonadIO m => a -> m ()
  putErrLn = hPutStrLn stderr

instance Print Text where
  hPutStr h = liftIO . Text.hPutStr h
  hPutStrLn h = liftIO . Text.hPutStrLn h

instance Print ByteString where
  hPutStr h = liftIO . ByteString.hPutStr h
  hPutStrLn h = liftIO . ByteString.hPutStrLn h

instance Print [Char] where
  hPutStr h = liftIO . SIO.hPutStr h
  hPutStrLn h = liftIO . SIO.hPutStrLn h

putText :: MonadIO m => Text -> m ()
putText = putStrLn

shout :: MonadIO m => Text -> m ()
shout = putErrLn
