module Init (exec) where

import Data.Bifunctor (first)
import OkStd
import System.Directory
import System.FilePath ((</>))
import System.IO (writeFile)

exec path =
  doesDirectoryExist path >>= \case
    False ->
      error $ "directory does not exist: " <> toText path
    True -> do
      shout $ "Creating website scaffold in '" <> toText path <> "'"
      for_ directories (createDirectoryIfMissing False . (path </>))
      for_ files (uncurry writeFile . first (path </>))
      shout "Done"
  where
    directories =
      ["build", "content", "static", "templates"]
    files =
      [ ("content/index.md", indexPage),
        ("templates/default.html", defaultTemplate)
      ]
    defaultTemplate =
      "<!doctype html><meta charset=\"utf-8\"><title>{{ _name }}</title><body>{{& _html }}</body>"
    indexPage =
      fromText $
        unlines
          [ "---",
            "{",
            "  \"title\": \"My very first page\",",
            "  \"key\": \"value\"",
            "}",
            "...",
            "",
            "This is my very first page"
          ]