{-# LANGUAGE TemplateHaskell #-}

module Main where

import qualified Build
import qualified Data.ByteString as BS
import Data.FileEmbed (embedFile)
import Data.Version (showVersion)
import qualified Init
import OkStd
import Paths_siteomat (version)
import System.Exit (exitWith)
import System.IO (hClose)
import System.Process (CreateProcess (..), StdStream (CreatePipe), proc, waitForProcess, withCreateProcess)

main :: IO ()
main =
  getArgs >>= \case
    ["init", path] -> Init.exec (fromText path)
    ["build", path] -> Build.exec (fromText path)
    ["help"] -> cmdHelp
    ["version"] -> cmdVersion
    [] -> cmdHelp
    args -> cmdNotFound $ unwords args

cmdHelp = do
  exitCode <- withCreateProcess manProc $ \(Just stdin) _ _ p -> do
    BS.hPut stdin manPage
    hClose stdin
    waitForProcess p
  exitWith exitCode
  where
    manProc =
      (proc "groff" ["-Tascii", "-man", "-"]) {delegate_ctlc = True, std_in = CreatePipe}
    manPage =
      $(embedFile "man/siteomat.1")

cmdVersion =
  putStrLn $ "Site-O-Mat " <> showVersion version

cmdNotFound cmd =
  error $ "invalid command: '" <> cmd <> "'" <> "\n(use 'siteomat help' for a list of commands and options)"